;###########################################################################
;###########################################################################
; ABOUT Menu:
;
;	This code module contains all of the functions that relate to 
; the menu that we use. 
;
;	There are routines for each menu we will have. One for the main
; menu and one for the load/save menu stuff.
;
;	NOTE: We could have combined these two functions into one generic 
; function that used parameters to determine the bahavior. But, by coding
; it explicitly we get a better idea for what is going on in the code.
;
;###########################################################################
;###########################################################################

;###########################################################################
;###########################################################################
; THE COMPILER OPTIONS
;###########################################################################
;###########################################################################

	.386
	.model flat, stdcall
	option casemap :none   ; case sensitive

;###########################################################################
;###########################################################################
; THE INCLUDES SECTION
;###########################################################################
;###########################################################################

	;================================================
	; These are the Inlcude files for Window stuff
	;================================================
	include \masm32\include\windows.inc
	include \masm32\include\comctl32.inc
	include \masm32\include\comdlg32.inc
	include \masm32\include\shell32.inc
	include \masm32\include\user32.inc
	include \masm32\include\kernel32.inc
	include \masm32\include\gdi32.inc
	
	;===============================================
	; The Lib's for those included files
	;================================================
	includelib \masm32\lib\comctl32.lib
	includelib \masm32\lib\comdlg32.lib
	includelib \masm32\lib\shell32.lib
	includelib \masm32\lib\gdi32.lib
	includelib \masm32\lib\user32.lib
	includelib \masm32\lib\kernel32.lib

	;====================================
	; The Direct Draw include file
	;====================================
	include Includes\DDraw.inc

	;====================================
	; The Direct Input include file
	;====================================
	include Includes\DInput.inc

	;====================================
	; The Direct Sound include file
	;====================================
	include Includes\DSound.inc

	;=================================================
	; Include the file that has our protos
	;=================================================
	include Protos.inc

;###########################################################################
;###########################################################################
; LOCAL MACROS
;###########################################################################
;###########################################################################

	m2m MACRO M1, M2
		push		M2
		pop		M1
	ENDM

	return MACRO arg
		mov	eax, arg
        	ret
	ENDM

;#################################################################################
;#################################################################################
; Variables we want to use in other modules
;#################################################################################
;#################################################################################


;#################################################################################
;#################################################################################
; External variables
;#################################################################################
;#################################################################################
	
	;=================================
	; The DirectDraw stuff
	;=================================
	EXTERN	lpddsprimary	:LPDIRECTDRAWSURFACE4
	EXTERN	lpddsback	:LPDIRECTDRAWSURFACE4

	;=========================================
	; The Input Device state variables
	;=========================================
	EXTERN keyboard_state	:BYTE

;#################################################################################
;#################################################################################
; BEGIN INITIALIZED DATA
;#################################################################################
;#################################################################################

    .data

	;===============================
	; Strings for the bitmaps
	;===============================
	szMainMenu	db "Art\Menu.sfp",0
	szFileMenu	db "Art\FileMenu.sfp",0

	;================================
	; Our very cool menu sound
	;================================
	szMenuSnd	db "Sound\Background.wav",0

	;===============================
	; PTR to the BMP's
	;===============================
	ptr_MAIN_MENU	dd 0
	ptr_FILE_MENU	dd 0

	;===============================
	; ID for the Menu sound
	;===============================
	Menu_ID		dd 0

	;======================================
	; A value to hold lPitch when locking
	;======================================
	lPitch		dd 0

	;========================================
	; Let's us know if we need to transition
	;========================================
	first_time	dd 0

;#################################################################################
;#################################################################################
; BEGIN CONSTANTS
;#################################################################################
;#################################################################################


;#################################################################################
;#################################################################################
; BEGIN EQUATES
;#################################################################################
;#################################################################################

	;=================
	;Utility Equates
	;=================
FALSE		equ	0
TRUE		equ	1

	;=================
	; The Screen BPP
	;=================
screen_bpp	equ	16

;=================
; The Menu Codes
;=================
	; Generic
MENU_ERROR	equ	0h
MENU_NOTHING	equ	1h

	; Main Menu
MENU_NEW	equ	2h
MENU_FILES	equ	3h
MENU_GAME	equ	4h
MENU_EXIT	equ	5h

	; File Menu
MENU_LOAD	equ	6h
MENU_SAVE	equ	7h
MENU_MAIN	equ	8h

;#################################################################################
;#################################################################################
; BEGIN THE CODE SECTION
;#################################################################################
;#################################################################################

  .code

;########################################################################
; Init_Menu Procedure
;########################################################################
Init_Menu	proc	

	;===========================================================
	; This function will initialize our menu systems
	;===========================================================

	;=================================
	; Local Variables
	;=================================

	;======================================
	; Read in the bitmap and create buffer
	;======================================
	invoke Create_From_SFP, ADDR ptr_MAIN_MENU, ADDR szMainMenu, screen_bpp

	;====================================
	; Test for an error
	;====================================
	.if eax == FALSE
		;========================
		; We failed so leave
		;========================
		jmp	err
	
	.endif

	;======================================
	; Read in the bitmap and create buffer
	;======================================
	invoke Create_From_SFP, ADDR ptr_FILE_MENU, ADDR szFileMenu, screen_bpp

	;====================================
	; Test for an error
	;====================================
	.if eax == FALSE
		;========================
		; We failed so leave
		;========================
		jmp	err
	
	.endif

	;========================
	; Load in the menu sound
	;========================
	invoke Load_WAV, ADDR szMenuSnd, NULL
	mov	Menu_ID, eax 

	;===============================
	; Set first_time to true so we 
	; will do a trans when we first 
	; enter the menu routines
	;===============================
	mov	first_time, TRUE

done:
	;===================
	; We completed
	;===================
	return TRUE

err:
	;===================
	; We didn't make it
	;===================
	return FALSE

Init_Menu	ENDP
;########################################################################
; END Init_Menu
;########################################################################

;########################################################################
; Shutdown_Menu Procedure
;########################################################################
Shutdown_Menu	proc	

	;===========================================================
	; This function will shutdown our menu systems
	;===========================================================

	;=================================
	; Local Variables
	;=================================

	;==========================
	; Free the bitmap memory
	;==========================
	invoke GlobalFree, ptr_MAIN_MENU
	invoke GlobalFree, ptr_FILE_MENU

done:
	;===================
	; We completed
	;===================
	return TRUE

err:
	;===================
	; We didn't make it
	;===================
	return FALSE

Shutdown_Menu	ENDP
;########################################################################
; END Shutdown_Menu
;########################################################################

;########################################################################
; Process_Main_Menu Procedure
;########################################################################
Process_Main_Menu	proc	

	;===========================================================
	; This function will process the main menu for the game
	;===========================================================

	;=================================
	; Local Variables
	;=================================

	;===================================
	; Lock the DirectDraw back buffer
	;===================================
	invoke DD_Lock_Surface, lpddsback, ADDR lPitch

	;============================
	; Check for an error
	;============================
	.if eax == FALSE
		;===================
		; Jump to err
		;===================
		jmp	err

	.endif

	;===================================
	; Draw the bitmap onto the surface
	;===================================
	invoke Draw_Bitmap, eax, ptr_MAIN_MENU, lPitch, 640, 480, screen_bpp

	;===================================
	; Unlock the back buffer
	;===================================
	invoke DD_Unlock_Surface, lpddsback

	;============================
	; Check for an error
	;============================
	.if eax == FALSE
		;===================
		; Jump to err
		;===================
		jmp	err

	.endif

	;======================================
	; Make sure the Menu sound is playing
	;======================================
	invoke Status_Sound, Menu_ID
	.if !(eax & DSBSTATUS_PLAYING)
		;===================
		; Play the sound
		;===================
		invoke Play_Sound, Menu_ID, DSBPLAY_LOOPING

	.endif

	;=====================================
	; Everything okay so flip displayed
	; surfaces and make loading visible
	; or call transition if needed
	;======================================
	.if first_time == TRUE
		invoke Transition
		mov	first_time, FALSE
	.else
		invoke DD_Flip
	.endif

	;============================
	; Check for an error
	;============================
	.if eax == FALSE
		;===================
		; Jump to err
		;===================
		jmp	err

	.endif

	;========================================================
	; Now read the keyboard to see if they have presses
	; any keys corresponding to our menu
	;========================================================
	invoke DI_Read_Keyboard

	;=============================
	; Did they press a valid key
	;=============================
	.if keyboard_state[DIK_N]
		;======================
		; Stop the menu music
		;======================
		invoke Stop_Sound, Menu_ID

		;===============================
		; Reset the first time variable
		;===============================
		mov	first_time, TRUE

		;======================
		; The new game key
		;======================
		return	MENU_NEW

	.elseif keyboard_state[DIK_G]
		;===============================
		; Reset the first time variable
		;===============================
		mov	first_time, TRUE

		;======================
		; The game files key
		;======================
		return MENU_FILES

	.elseif keyboard_state[DIK_R]
		;===============================
		; Reset the first time variable
		;===============================
		mov	first_time, TRUE

		;======================
		; Stop the menu music
		;======================
		invoke Stop_Sound, Menu_ID

		;======================
		; Return to game key
		;======================
		return MENU_GAME

	.elseif keyboard_state[DIK_E]
		;======================
		; Stop the menu music
		;======================
		invoke Stop_Sound, Menu_ID

		;======================
		; The exit game key
		;======================
		return MENU_EXIT

	.endif

done:
	;===================
	; We completed w/o
	; doing anything
	;===================
	return MENU_NOTHING

err:
	;===================
	; We didn't make it
	;===================
	return MENU_ERROR

Process_Main_Menu	ENDP
;########################################################################
; END Process_Main_Menu
;########################################################################

;########################################################################
; Process_File_Menu Procedure
;########################################################################
Process_File_Menu	proc	

	;===========================================================
	; This function will process the file menu for the gane
	;===========================================================

	;=================================
	; Local Variables
	;=================================

	;===================================
	; Lock the DirectDraw back buffer
	;===================================
	invoke DD_Lock_Surface, lpddsback, ADDR lPitch

	;============================
	; Check for an error
	;============================
	.if eax == FALSE
		;===================
		; Jump to err
		;===================
		jmp	err

	.endif

	;===================================
	; Draw the bitmap onto the surface
	;===================================
	invoke Draw_Bitmap, eax, ptr_FILE_MENU, lPitch, 640, 480, screen_bpp

	;===================================
	; Unlock the back buffer
	;===================================
	invoke DD_Unlock_Surface, lpddsback

	;============================
	; Check for an error
	;============================
	.if eax == FALSE
		;===================
		; Jump to err
		;===================
		jmp	err

	.endif

	;======================================
	; Make sure the Menu sound is playing
	;======================================
	invoke Status_Sound, Menu_ID
	.if !(eax & DSBSTATUS_PLAYING)
		;===================
		; Play the sound
		;===================
		invoke Play_Sound, Menu_ID, DSBPLAY_LOOPING

	.endif

	;=====================================
	; Everything okay so flip displayed
	; surfaces and make loading visible
	; or call transition if needed
	;======================================
	.if first_time == TRUE
		invoke Transition
		mov	first_time, FALSE
	.else
		invoke DD_Flip
	.endif

	;============================
	; Check for an error
	;============================
	.if eax == FALSE
		;===================
		; Jump to err
		;===================
		jmp	err

	.endif


	;========================================================
	; Now read the keyboard to see if they have presses
	; any keys corresponding to our menu
	;========================================================
	invoke DI_Read_Keyboard

	;=============================
	; Did they press a valid key
	;=============================
	.if keyboard_state[DIK_L]
		;======================
		; The load game key
		;======================
		return	MENU_LOAD

	.elseif keyboard_state[DIK_S]
		;======================
		; The save game key
		;======================
		return MENU_SAVE

	.elseif keyboard_state[DIK_B]
		;===============================
		; Reset the first time variable
		;===============================
		mov	first_time, TRUE

		;======================
		; Return to main key
		;======================
		return MENU_MAIN

	.endif

done:
	;===================
	; We completed w/o
	; doing anything
	;===================
	return MENU_NOTHING

err:
	;===================
	; We didn't make it
	;===================
	return MENU_ERROR

Process_File_Menu	ENDP
;########################################################################
; END Process_File_Menu
;########################################################################

;######################################
; THIS IS THE END OF THE PROGRAM CODE #
;######################################
end 

