;###########################################################################
;###########################################################################
; ABOUT SPACE-TRIS:
;
;	This is the main portion of code. It has WinMain and performs all
;	of the management for the game.
;
;		- WinMain()
;		- WndProc()
;		- Game_Init()
;		- Game_Main()
;		- Game_Shutdown()
;
;
;###########################################################################
;###########################################################################


;###########################################################################
;###########################################################################
; THE COMPILER OPTIONS
;###########################################################################
;###########################################################################

	.386
	.model flat, stdcall
	option casemap :none   ; case sensitive

;###########################################################################
;###########################################################################
; THE INCLUDES SECTION
;###########################################################################
;###########################################################################
	
	;================================================
	; These are the Inlcude files for Window calls
	;================================================
	include \masm32\include\windows.inc
	include \masm32\include\comctl32.inc
	include \masm32\include\comdlg32.inc
	include \masm32\include\shell32.inc
	include \masm32\include\user32.inc
	include \masm32\include\kernel32.inc
	include \masm32\include\gdi32.inc

	;====================================
	; The Direct Draw include file
	;====================================
	include Includes\DDraw.inc
	
	;====================================
	; The Direct Input include file
	;====================================
	include Includes\DInput.inc

	;====================================
	; The Direct Sound include file
	;====================================
	include Includes\DSound.inc

	;===============================================
	; The Lib's for those included files
	;================================================
	includelib \masm32\lib\comctl32.lib
	includelib \masm32\lib\comdlg32.lib
	includelib \masm32\lib\shell32.lib
	includelib \masm32\lib\gdi32.lib
	includelib \masm32\lib\user32.lib
	includelib \masm32\lib\kernel32.lib

	;=================================================
	; Include the file that has our prototypes
	;=================================================
	include Protos.inc

;###########################################################################
;###########################################################################
; LOCAL MACROS
;###########################################################################
;###########################################################################

	szText MACRO Name, Text:VARARG
		LOCAL lbl
		jmp lbl
		Name db Text,0
		lbl:
	ENDM

	m2m MACRO M1, M2
		push		M2
		pop		M1
	ENDM

	return MACRO arg
		mov	eax, arg
        	ret
	ENDM

	RGB MACRO red, green, blue
		xor	eax,eax
		mov	ah,blue
		shl	eax,8
		mov	ah,green
		mov	al,red
	ENDM

;#################################################################################
;#################################################################################
; Variables we want to use in other modules
;#################################################################################
;#################################################################################

	;===========================================
	; The main window handle
	;===========================================
	PUBLIC	hMainWnd
	PUBLIC	hInst

;#################################################################################
;#################################################################################
; External variables
;#################################################################################
;#################################################################################

	;=================================
	; The DirectDraw stuff
	;=================================
	EXTERN	lpddsprimary	:LPDIRECTDRAWSURFACE4
	EXTERN	lpddsback	:LPDIRECTDRAWSURFACE4

	;=========================================
	; The Input Device state variables
	;=========================================
	EXTERN keyboard_state	:BYTE

;#################################################################################
;#################################################################################
; BEGIN INITIALIZED DATA
;#################################################################################
;#################################################################################

    .data
	;==============================
	;Text for the Window Title and
	; the class name
	;==============================
    	szDisplayName	db "SPACE-TRIS",0
	szClassName	db "ST_Class",0
	
	;==============================
	;Windows handles and Misc
	;==============================
	msg  MSG	<?>	; For message handling
	CommandLine	dd 0	; for the commandline params
	hMainWnd	dd 0	; Handle to the main window
	hInst		dd 0	; Handle to an Instance

	;======================================
	; A variable to hold the game state
	;======================================
	GameState	db 0

	;======================================
	; A value to hold lPitch when locking
	;======================================
	lPitch		dd 0

	;===============================
	; Strings for the Loading bitmap
	;===============================
	szLoading		db "Art\Loading.sfp",0
	szMain			db "Art\Main.sfp",0

	;===============================
	; PTR to the BMP's
	;===============================
	ptr_BMP_LOAD	dd 0
	ptr_BMP_MAIN	dd 0

	;================================
	; Our very cool background music
	;================================
	szMusicSnd	db "Sound\Music.wav",0

	;===============================
	; ID for the background Music
	;===============================
	Music_ID	dd 0

	;================================
	; Timer Variables
	;================================
	Input_Time	dd 0
	Update_Time	dd 0

;#################################################################################
;#################################################################################
; BEGIN CONSTANTS
;#################################################################################
;#################################################################################


;#################################################################################
;#################################################################################
; BEGIN EQUATES
;#################################################################################
;#################################################################################

	;=================
	;Utility Equates
	;=================
FALSE		equ	0
TRUE		equ	1

	;=================
	; The Screen BPP
	;=================
screen_bpp	equ	16

	;================
	; resource IDs
	;================
IDI_ICON	equ	01h		


	;======================================
	; Set the default sync rate - 25 FPS
	;======================================
sync_time	equ	40	; in Milliseconds

	;=====================
	; For the game state
	;=====================
GS_MENU		equ	01h
GS_EXIT		equ	02h
GS_DIE		equ	03h
GS_FILE		equ	04h
GS_PLAY		equ	05h

;=================
; The Menu Codes
;=================
	; Generic
MENU_ERROR	equ	0h
MENU_NOTHING	equ	1h

	; Main Menu
MENU_NEW	equ	2h
MENU_FILES	equ	3h
MENU_GAME	equ	4h
MENU_EXIT	equ	5h

	; File Menu
MENU_LOAD	equ	6h
MENU_SAVE	equ	7h
MENU_MAIN	equ	8h

	;=================
	; Movement EQU's
	;=================
MOVE_LEFT	equ	1
MOVE_RIGHT	equ	2
MOVE_DOWN	equ	3

	;===================
	; Millisecond delays
	;===================
INPUT_DELAY	equ	100
UPDATE_DELAY	equ	250

;#################################################################################
;#################################################################################
; BEGIN THE CODE SECTION
;#################################################################################
;#################################################################################

  .code

start:
	;==================================
	; Obtain the instance for the
	; application
	;==================================
	invoke GetModuleHandle, NULL
	mov	hInst, eax

	;==================================
	; Is there a commandline to parse?
	;==================================
	invoke GetCommandLine
	mov	CommandLine, eax

	;==================================
	; Call the WinMain procedure
	;==================================
	invoke WinMain,hInst,NULL,CommandLine,SW_SHOWDEFAULT

	;==================================
	; Leave the program
	;==================================
	invoke ExitProcess,eax


;########################################################################
; WinMain Function
;########################################################################
WinMain proc	hInstance	:DWORD,
		hPrevInst	:DWORD,
		CmdLine		:DWORD,
		CmdShow		:DWORD

	;====================
	; Put LOCALs on stack
	;====================
	LOCAL wc		:WNDCLASS

	;==================================================
	; Fill WNDCLASS structure with required variables
	;==================================================
	mov	wc.style, CS_OWNDC
	mov	wc.lpfnWndProc,offset WndProc
	mov	wc.cbClsExtra,NULL
	mov	wc.cbWndExtra,NULL
	m2m	wc.hInstance,hInst   ;<< NOTE: macro not mnemonic
	invoke GetStockObject, BLACK_BRUSH
	mov	wc.hbrBackground, eax
	mov	wc.lpszMenuName,NULL
	mov	wc.lpszClassName,offset szClassName
	invoke LoadIcon, hInst, IDI_ICON ; icon ID
	mov	wc.hIcon,eax
	invoke LoadCursor,NULL,IDC_ARROW
	mov	wc.hCursor,eax

	;================================
	; Register our class we created
	;================================
	invoke RegisterClass, ADDR wc

	;===========================================
	; Create the main screen
	;===========================================
	invoke CreateWindowEx,NULL,
			ADDR szClassName,
                        ADDR szDisplayName,
                        WS_POPUP or WS_CLIPSIBLINGS or \
			WS_MAXIMIZE or WS_CLIPCHILDREN,
                        0,0,640,480,
                        NULL,NULL,
                        hInst,NULL
        
	;===========================================
	; Put the window handle in for future uses 
	;===========================================
  	mov	hMainWnd, eax

	;====================================
	; Hide the cursor
	;====================================
	invoke ShowCursor, FALSE

	;===========================================
	; Display our Window we created for now
	;===========================================
	invoke ShowWindow, hMainWnd, SW_SHOWDEFAULT

	;=================================
	; Intialize the Game
	;=================================
	invoke Game_Init

	;========================================
	; Check for an error if so leave
	;========================================
	.if eax != TRUE
		jmp	shutdown
	.endif

	;===================================
	; Loop until PostQuitMessage is sent
	;===================================
  	.WHILE TRUE
		invoke	PeekMessage, ADDR msg, NULL, 0, 0, PM_REMOVE
		.if (eax != 0)
			;===================================
			; Break if it was the quit messge
			;===================================
			mov	eax, msg.message
			.IF eax == WM_QUIT
				;======================
				; Break out
				;======================
				jmp	shutdown
			.endif

			;===================================
			; Translate and Dispatch the message
			;===================================
			invoke	TranslateMessage, ADDR msg
			invoke	DispatchMessage, ADDR msg

		.endif

		;================================
		; Call our Main Game Loop 
		;
		; NOTE: This is done every loop
		; iteration no matter what
		;================================
		invoke Game_Main

		;=============================
		; Do they want to leave
		;=============================
		.if GameState == GS_EXIT
			jmp	shutdown
		.endif

	.ENDW

shutdown:

	;=================================
	; Shutdown the Game
	;=================================
	invoke Game_Shutdown

	;=================================
	; Show the Cursor
	;=================================
	invoke ShowCursor, TRUE

getout:
	;===========================
	; We are through 
	;===========================
	return msg.wParam

WinMain endp
;########################################################################
; End of WinMain Procedure
;########################################################################



;########################################################################
; Main Window Callback Procedure -- WndProc
;########################################################################
WndProc proc	hWin   :DWORD,
		uMsg   :DWORD,
		wParam :DWORD,
		lParam :DWORD

.if uMsg == WM_COMMAND
	;===========================
	; We don't have a menu, but 
	; if we did this is where it
	; would go!
	; NOTE: This means an app
	; menu not our bitmap menu!
	;===========================

.elseif uMsg == WM_DESTROY
	;===========================
	; Kill the application
	;===========================
	invoke PostQuitMessage,NULL
 	return 0 
 
.endif

;=================================================
; Let the default procedure handle the message
;=================================================
invoke DefWindowProc,hWin,uMsg,wParam,lParam

ret

WndProc endp
;########################################################################
; End of Main Windows Callback Procedure
;########################################################################



;========================================================================
;========================================================================
; THE GAME PROCEDURES
;========================================================================
;========================================================================


;########################################################################
; Game_Init Procedure
;########################################################################
Game_Init	proc

	;=========================================================
	; This function will setup the game
	;=========================================================
	
	;============================================
	; Initialize Direct Draw -- 640, 480, bpp
	;============================================
	invoke DD_Init, 640, 480, screen_bpp

	;====================================
	; Test for an error
	;====================================
	.if eax == FALSE
		;========================
		; We failed so leave
		;========================
		jmp	err
	
	.endif

	;======================================
	; Read in the bitmap and create buffer
	;======================================
	invoke Create_From_SFP, ADDR ptr_BMP_LOAD, ADDR szLoading, screen_bpp

	;====================================
	; Test for an error
	;====================================
	.if eax == FALSE
		;========================
		; We failed so leave
		;========================
		jmp	err
	
	.endif

	;===================================
	; Lock the DirectDraw back buffer
	;===================================
	invoke DD_Lock_Surface, lpddsback, ADDR lPitch

	;============================
	; Check for an error
	;============================
	.if eax == FALSE
		;===================
		; Jump to err
		;===================
		jmp	err

	.endif

	;===================================
	; Draw the bitmap onto the surface
	;===================================
	invoke Draw_Bitmap, eax, ptr_BMP_LOAD, lPitch, 640, 480, screen_bpp

	;===================================
	; Unlock the back buffer
	;===================================
	invoke DD_Unlock_Surface, lpddsback

	;============================
	; Check for an error
	;============================
	.if eax == FALSE
		;===================
		; Jump to err
		;===================
		jmp	err

	.endif

	;=====================================
	; Everything okay so flip displayed
	; surfaces and make loading visible
	;======================================
	invoke DD_Flip

	;============================
	; Check for an error
	;============================
	.if eax == FALSE
		;===================
		; Jump to err
		;===================
		jmp	err

	.endif

	;======================================
	; Read in main bitmap and create buffer
	;======================================
	invoke Create_From_SFP, ADDR ptr_BMP_MAIN, ADDR szMain, screen_bpp

	;====================================
	; Test for an error
	;====================================
	.if eax == FALSE
		;========================
		; We failed so leave
		;========================
		jmp	err
	
	.endif

	;======================================
	; Initialize Direct Input
	;======================================
	invoke DI_Init

	;====================================
	; Test for an error
	;====================================
	.if eax == FALSE
		;========================
		; We failed so leave
		;========================
		jmp	err
	
	.endif

	;======================================
	; Initialize Direct Sound
	;======================================
	invoke DS_Init

	;====================================
	; Test for an error
	;====================================
	.if eax == FALSE
		;========================
		; We failed so leave
		;========================
		jmp	err
	
	.endif

	;========================
	; Load in the level music
	;========================
	invoke Load_WAV, ADDR szMusicSnd, NULL
	mov	Music_ID, eax 

	;========================================
	; Initialize the timing system
	;========================================
	invoke Init_Time

	;======================================
	; Initialize Our Shapes
	;======================================
	invoke Init_Shapes

	;====================================
	; Test for an error
	;====================================
	.if eax == FALSE
		;========================
		; We failed so leave
		;========================
		jmp	err
	
	.endif

	;======================================
	; Initialize Our Grid
	;======================================
	invoke Init_Grid

	;====================================
	; Test for an error
	;====================================
	.if eax == FALSE
		;========================
		; We failed so leave
		;========================
		jmp	err
	
	.endif

	;=============================
	; Get a Starting Shape
	;=============================
	invoke New_Shape

	;======================================
	; Initialize Our Menus
	;======================================
	invoke Init_Menu

	;====================================
	; Test for an error
	;====================================
	.if eax == FALSE
		;========================
		; We failed so leave
		;========================
		jmp	err
	
	.endif

	;===================================
	; Set the game state to the menu
	; state since that is our fist stop
	;===================================
	mov	GameState, GS_MENU

	;==========================
	; Free the bitmap memory
	;==========================
	invoke GlobalFree, ptr_BMP_LOAD

done:
	;===================
	; We completed
	;===================
	return TRUE

err:
	;===================
	; We didn't make it
	;===================
	return FALSE

Game_Init	ENDP
;########################################################################
; END Game_Init
;########################################################################



;########################################################################
; Game_Main Procedure
;########################################################################
Game_Main	proc

	;============================================================
	; This is the heart of the game it gets called over and over
	; and even if we process a message!
	;============================================================

	;=========================================
	; Local Variables
	;=========================================
	LOCAL	StartTime	:DWORD

	;====================================
	; Get the starting time for the loop
	;====================================
	invoke Start_Time, ADDR StartTime

	;==============================================================
	; Take the proper action(s) based on the GameState variable
	;==============================================================
	.if GameState == GS_MENU
		;=================================
		; We are in the main menu state
		;=================================
		invoke Process_Main_Menu
		
		;=================================
		; What did they want to do
		;=================================
		.if eax == MENU_NOTHING
			;=================================
			; They didn't select anything yet
			; so don't do anything 
			;=================================

		.elseif eax == MENU_ERROR
			;==================================
			; This is where error code would go
			;==================================

		.elseif eax == MENU_NEW
			;==================================
			; They want to start a new game
			;==================================

			;=============================
			; Re-Init the grid
			;=============================
			invoke Init_Grid

			;=============================
			; Get a new Starting Shape
			;=============================
			invoke New_Shape

			;====================================
			; Get starting time for the input
			;====================================
			invoke Get_Time
			mov	Input_Time, eax

			;====================================
			; Get starting time for the updates
			;====================================
			invoke Get_Time
			mov	Update_Time, eax

			;===============================
			; Set the Game state to playing
			;===============================
			mov	GameState, GS_PLAY

		.elseif eax == MENU_FILES
			;==================================
			; They want the file menu
			;==================================
			mov	GameState, GS_FILE

		.elseif eax == MENU_GAME
			;==================================
			; They want to return to the game
			;==================================

			;===============================
			; Set the Game state to playing
			;===============================
			mov	GameState, GS_PLAY

		.elseif eax == MENU_EXIT
			;==================================
			; Transition to black
			;==================================
			invoke DD_Fill_Surface, lpddsback, 0, 0, 0
			invoke Transition

			;==================================
			; They want to exit the game
			;==================================
			mov	GameState, GS_EXIT

		.endif


	.elseif GameState == GS_FILE
		;=================================
		; We are in the file menu state
		;=================================
		invoke Process_File_Menu
		
		;=================================
		; What did they want to do
		;=================================
		.if eax == MENU_NOTHING
			;=================================
			; They didn't select anything yet
			; so don't do  anything 
			;=================================

		.elseif eax == MENU_ERROR
			;==================================
			; This is where error code would go
			;==================================

		.elseif eax == MENU_LOAD
			;==================================
			; They want to load game
			;==================================

		.elseif eax == MENU_SAVE
			;==================================
			; They want to save their game
			;==================================


		.elseif eax == MENU_MAIN
			;==================================
			; They want to return to main menu
			;==================================
			mov	GameState, GS_MENU

		.endif


	.elseif GameState == GS_PLAY
		;=================================
		; We are in the gameplay mode
		;=================================

		;======================================
		; Make sure the level music is playing
		;======================================
		invoke Status_Sound, Music_ID
		.if !(eax & DSBSTATUS_PLAYING)
			;===================
			; Play the sound
			;===================
			invoke Play_Sound, Music_ID, DSBPLAY_LOOPING

		.endif

		;===============================
		; Load the main bitmap into the
		; back buffer
		;===============================
		invoke DD_Load_Bitmap, lpddsback, ptr_BMP_MAIN, \
			640, 480, screen_bpp

		;=====================================
		; Is it time to process input yet?
		;=====================================
		invoke Get_Time
		sub	eax, INPUT_DELAY
		.if eax  > Input_Time
			;==================
			; It is time.
			;==================
		
			;========================================
			; Read the Keyboard
			;========================================
			invoke DI_Read_Keyboard

			;================================
			; What do they want to do
			;================================
			.if keyboard_state[DIK_ESCAPE]
				;=========================
				; Stop the music
				;=========================
				invoke Stop_Sound, Music_ID

				;========================
				; The return to menu key
				;========================
				mov	GameState, GS_MENU

			.elseif keyboard_state[DIK_UP]
				;======================
				; Rotate the shape
				;======================
				invoke Rotate_Shape

			.elseif keyboard_state[DIK_DOWN]
				;======================
				; Move the shape down
				;======================
				invoke Move_Shape, MOVE_DOWN

			.elseif keyboard_state[DIK_LEFT]
				;======================
				; Move the shape left
				;======================
				invoke Move_Shape, MOVE_LEFT

			.elseif keyboard_state[DIK_RIGHT]
				;======================
				; Move the shape Right
				;======================
				invoke Move_Shape, MOVE_RIGHT

			.endif

			;============================
			; Get a New Input Time
			;============================
			invoke Get_Time
			mov	Input_Time, eax

		.endif

		;=====================================
		; Is it time to update the shape yet?
		;=====================================
		invoke Get_Time
		sub	eax, UPDATE_DELAY
		.if eax  > Update_Time
			;==================
			; It is time.
			;==================

			;===============================
			; Update the current shape
			;===============================
			invoke Update_Shape

			;===============================
			; Did we not succeed at updating
			;===============================
			.if eax == FALSE
				;=======================
				; They had a collision
				;=======================

				;=======================
				; Test for a line
				;=======================
				invoke Line_Test

				;=======================
				; Did they earn one?
				;=======================
				.while eax == TRUE
					;================
					; They got one
					;================

					;=================
					; Test for another
					;=================
					invoke Line_Test

				.endw

				;=======================
				; Start a new piece
				;=======================
				invoke New_Shape

				;=======================
				; Did we make it?
				;=======================
				.if eax == FALSE
					;===============
					; They died!
					;===============
					mov	GameState, GS_DIE

				.endif


			.endif

			;============================
			; Get a New Update Time
			;============================
			invoke Get_Time
			mov	Update_Time, eax

		.endif

		;===============================
		; Draw our current grid
		;===============================
		invoke Draw_Grid

		;===============================
		; Draw our current shape
		;===============================
		invoke Draw_Shape

		;===============================
		; Flip the buffers
		;===============================
		invoke DD_Flip

	.elseif GameState == GS_DIE
		;=================================
		; We died so perform that code
		;=================================

		;=================================
		; Wait for a couple of seconds so
		; they know that they have died
		;=================================
		invoke Sleep, 2000

		;=================================
		; ReInit the Grid
		;=================================
		invoke Init_Grid

		;=================================
		; Get a New shape
		;=================================
		invoke New_Shape

		;=================================
		; Back to the Main Menu
		;=================================
		mov	GameState, GS_MENU

	.endif

	;===================================
	; Wait to synchronize the time
	;===================================
	invoke Wait_Time, StartTime, sync_time

done:
	;===================
	; We completed
	;===================
	return TRUE

err:
	;===================
	; We didn't make it
	;===================
	return FALSE

Game_Main	ENDP
;########################################################################
; END Game_Main
;########################################################################



;########################################################################
; Game_Shutdown Procedure
;########################################################################
Game_Shutdown	proc

	;============================================================
	; This shuts our game down and frees memory we allocated
	;============================================================

	;===========================
	; Shutdown the shapes
	;===========================
	invoke Shutdown_Shapes

	;==========================
	; Free the bitmap memory
	;==========================
	invoke GlobalFree, ptr_BMP_MAIN

	;===========================
	; Shutdown the menus
	;===========================
	invoke Shutdown_Menu

	;===========================
	; Shutdown Direct Sound
	;===========================
	invoke DS_ShutDown

	;===========================
	; Shutdown Direct Input
	;===========================
	invoke DI_ShutDown

	;===========================
	; Shutdown DirectDraw
	;===========================
	invoke DD_ShutDown

done:
	;===================
	; We completed
	;===================
	return TRUE

err:
	;===================
	; We didn't make it
	;===================
	return FALSE

Game_Shutdown	ENDP
;########################################################################
; END Game_Shutdown
;########################################################################


;######################################
; THIS IS THE END OF THE PROGRAM CODE #
;######################################
end start
